@@ -1,4 +1,4 @@
-/* $Id: jigdo-io.cc,v 1.15 2004/06/20 20:35:15 atterer Exp $ -*- C++ -*-
+/* $Id: jigdo-io.cc,v 1.26 2005/04/10 16:36:31 atterer Exp $ -*- C++ -*-
   __   _
   |_) /|  Copyright (C) 2003  |  richard@
   | \/|  Richard Atterer     |  atterer.net
@@ -22,7 +22,7 @@
 #include <makeimagedl.hh>
 #include <md5sum.hh>
 #include <mimestream.hh>
-//#include <url-part.hh>
+#include <uri.hh>
 //______________________________________________________________________
 
 DEBUG_UNIT("jigdo-io")
@@ -45,75 +45,46 @@ namespace {
 }
 
 // Root object
-JigdoIO::JigdoIO(MakeImageDl::Child* c, const string& url,
-                 DataSource::IO* frontendIo)
-  : childDl(c), urlVal(url), frontend(frontendIo), parent(0), includeLine(0),
+JigdoIO::JigdoIO(MakeImageDl::Child* c, const string& url/*,
+                 DataSource::IO* frontendIo*/)
+  : childDl(c), urlVal(url),/*frontend(frontendIo),*/parent(0), includeLine(0),
     firstChild(0), next(0), rootAndImageSectionCandidate(this), line(0),
     section(), imageSectionLine(0), imageName(), imageInfo(),
-    imageShortInfo(), templateUrl(), templateMd5(0), childFailedId(0),
-    gunzip(this) { }
+    imageShortInfo(), templateUrls(), templateMd5(0), /*childFailedId(0),*/
+    gunzip(this) {
+  debug("JigdoIO %1", this);
+}
 
 // Non-root, i.e. [Include]d object
 JigdoIO::JigdoIO(MakeImageDl::Child* c, const string& url,
-                 DataSource::IO* frontendIo, JigdoIO* parentJigdo,
+                 /*DataSource::IO* frontendIo,*/ JigdoIO* parentJigdo,
                  int inclLine)
-  : childDl(c), urlVal(url), frontend(frontendIo), parent(parentJigdo),
+  : childDl(c), urlVal(url), /*frontend(frontendIo),*/ parent(parentJigdo),
     includeLine(inclLine), firstChild(0), next(0),
     rootAndImageSectionCandidate(parent->root()), line(0), section(),
     imageSectionLine(0), imageName(), imageInfo(), imageShortInfo(),
-    templateUrl(), templateMd5(0), childFailedId(0), gunzip(this) {
-  //debug("JigdoIO: Parent of %1 is %2", url, parent->urlVal);
+    templateUrls(), templateMd5(0), /*childFailedId(0),*/ gunzip(this) {
+  debug("JigdoIO %1: Parent of %2 is %3", this, url, parent->urlVal);
 }
 //______________________________________________________________________
 
 JigdoIO::~JigdoIO() {
   debug("~JigdoIO");
 
-  if (childFailedId != 0) {
-    g_source_remove(childFailedId);
-    childFailedId = 0;
-    master()->childFailed(childDl, this, frontend);
+  // Delete all our children
+  JigdoIO* x = firstChild;
+  while (x != 0) {
+    JigdoIO* y = x->next;
+    debug("~JigdoIO: deleting child %1", x);
+    delete x;
+    x = y;
   }
 
-  /* Don't delete children; master will do this! If we deleted them here,
-     MakeImageDl::Child::childIoVal would be left dangling. */
-//   // Delete all our children
-//   JigdoIO* x = firstChild;
-//   while (x != 0) {
-//     JigdoIO* y = x->next;
-//     delete x;
-//     x = y;
-//   }
-
   delete templateMd5;
-
-  if (source() != 0) {
-    source()->io.remove(this);
-    Paranoid(source()->io.get() != this);
-  }
 }
 //______________________________________________________________________
 
-Job::IO* JigdoIO::job_removeIo(Job::IO* rmIo) {
-  debug("job_removeIo %1", rmIo);
-  if (rmIo == this) {
-    // Do not "delete this" - this is called from ~JigdoIO above
-    DataSource::IO* c = frontend;
-    frontend = 0;
-    return c;
-  } else if (frontend != 0) {
-    Job::IO* c = frontend->job_removeIo(rmIo);
-    Paranoid(c == 0 || dynamic_cast<DataSource::IO*>(c) != 0);
-    debug("job_removeIo frontend=%1", c);
-    frontend = static_cast<DataSource::IO*>(c);
-  }
-  return this;
-}
-
-void JigdoIO::job_deleted() {
-  if (frontend != 0) frontend->job_deleted();
-  // Do not "delete this" - childDl owns us
-}
+void JigdoIO::job_deleted() { }
 
 void JigdoIO::job_succeeded() {
   if (failed()) return;
@@ -133,38 +104,24 @@ void JigdoIO::job_succeeded() {
   setFinished();
   XStatus st = imgSect_eof();
   if (st.xfailed()) return;
-  if (frontend != 0) frontend->job_succeeded();
-  master()->childSucceeded(childDl, this, frontend);
+  //master()->childSucceeded(childDl, this);
   if (st.returned(1)) master()->jigdoFinished(); // Causes "delete this"
 }
 
-void JigdoIO::job_failed(string* message) {
-  Paranoid(!failed());
+void JigdoIO::job_failed(const string&) {
   if (failed()) return;
-  if (frontend != 0) frontend->job_failed(message);
   string err = _("Download of .jigdo file failed");
-  master()->generateError(&err);
-  /* We cannot call this right now:
-     master()->childFailed(childDl, this, frontend);
-     so schedule a callback to call it later. */
-  childFailedId = g_idle_add_full(G_PRIORITY_HIGH_IDLE,&childFailed_callback,
-                                  (gpointer)this, NULL);
-  Paranoid(childFailedId != 0);
+  master()->generateError(err);
   imageName.assign("", 1); Paranoid(failed());
+
+  //master()->childFailed(childDl, this);
 }
 
-void JigdoIO::job_message(string* message) {
-  if (failed()) return;
-  if (frontend != 0) frontend->job_message(message);
-}
+void JigdoIO::job_message(const string&) { }
 
-void JigdoIO::dataSource_dataSize(uint64 n) {
-  if (failed()) return;
-  if (frontend != 0) frontend->dataSource_dataSize(n);
-}
+void JigdoIO::dataSource_dataSize(uint64) { }
 
-void JigdoIO::dataSource_data(const byte* data, size_t size,
-                              uint64 currentSize) {
+void JigdoIO::dataSource_data(const byte* data, unsigned size, uint64) {
   Assert(!finished());
   if (/*master()->finalState() ||*/ failed()) {
     debug("Got %1 bytes, ignoring", size);
@@ -179,7 +136,6 @@ void JigdoIO::dataSource_data(const byte* data, size_t size,
     generateError(e.message);
     return;
   }
-  if (frontend != 0) frontend->dataSource_data(data, size, currentSize);
 }
 //______________________________________________________________________
 
@@ -263,7 +219,7 @@ void JigdoIO::generateError(const string& msg) {
                      _("%1 (at end of %3)") : _("%1 (line %2 in %3)"));
   err = subst(fmt, msg, line,
               (source() != 0 ? source()->location().c_str() : "?") );
-  generateError_plain(&err);
+  generateError_plain(err);
 }
 
 void JigdoIO::generateError(const char* msg) {
@@ -272,34 +228,18 @@ void JigdoIO::generateError(const char* msg) {
                      _("%1 (at end of %3)") : _("%1 (line %2 in %3)"));
   err = subst(fmt, msg, line,
               (source() != 0 ? source()->location().c_str() : "?") );
-  generateError_plain(&err);
+  generateError_plain(err);
 }
 
-void JigdoIO::generateError_plain(string* err) {
+void JigdoIO::generateError_plain(const string& err) {
   debug("generateError: %1", err);
   Paranoid(!failed());
   if (failed()) return;
-  if (frontend != 0) frontend->job_failed(err);
-  *err = _("Error processing .jigdo file contents");
-  master()->generateError(err);
-
-  /* We cannot call this right now:
-     master()->childFailed(childDl, this, frontend);
-     so schedule a callback to call it later. */
-  childFailedId = g_idle_add_full(G_PRIORITY_HIGH_IDLE,&childFailed_callback,
-                                  (gpointer)this, NULL);
-  Paranoid(childFailedId != 0);
+  IOSOURCE_SEND(Job::DataSource::IO, source()->io, job_failed, (err));
+  string merr = _("Error processing .jigdo file contents");
+  master()->generateError(merr);
   imageName.assign("", 1); Paranoid(failed());
-}
-
-gboolean JigdoIO::childFailed_callback(gpointer data) {
-  JigdoIO* self = static_cast<JigdoIO*>(data);
-  debug("childFailed_callback for %1",
-        (self->source() != 0 ? self->source()->location().c_str() : "?") );
-  self->childFailedId = 0;
-  self->master()->childFailed(self->childDl, self, self->frontend);
-  self->master()->jigdoFinished(); // "delete self"
-  return FALSE; // "Don't call me again"
+  master()->childFailed(childDl);
 }
 //______________________________________________________________________
 
@@ -352,7 +292,7 @@ void JigdoIO::imgSect_parsed() {
         ? "(haveImageSection)" : ""), urlVal, line - 1);
   if (master()->haveImageSection()) return;
   master()->setImageSection(&imageName, &imageInfo, &imageShortInfo,
-                            &templateUrl, &templateMd5);
+                            templateUrls.get(), &templateMd5);
 }
 
 #if DEBUG
@@ -395,7 +335,7 @@ XStatus JigdoIO::imgSect_eof() {
         debug("imgSect_eof:%1%2Found before [Include]", have(m), indent);
         if (!m->haveImageSection())
           m->setImageSection(&x->imageName, &x->imageInfo,
-              &x->imageShortInfo, &x->templateUrl, &x->templateMd5);
+              &x->imageShortInfo, x->templateUrls.get(), &x->templateMd5);
       }
       // No [Image] inbetween - move on, descend into [Include]
       debug("imgSect_eof:%1%2Now at %3:%4, descending",
@@ -411,8 +351,8 @@ XStatus JigdoIO::imgSect_eof() {
       debug("imgSect_eof:%1%2Found after last [Include], if any",
             have(m), indent);
       if (!m->haveImageSection())
-        m->setImageSection(&x->imageName, &x->imageInfo,
-                       &x->imageShortInfo, &x->templateUrl, &x->templateMd5);
+        m->setImageSection(&x->imageName, &x->imageInfo, &x->imageShortInfo,
+                           x->templateUrls.get(), &x->templateMd5);
     }
 
     // Nothing found. If x not yet fully downloaded, stop here
@@ -463,9 +403,6 @@ void JigdoIO::jigdoLine(string* l) {
       return generateError(_("No `=' after first word"));
     ++x; // Skip '='
     advanceWhitespace(x, end);
-//     vector<string> value;
-//     ConfigFile::split(value, s, x - s.begin());
-//     entry(&labelName, &value);
     entry(&labelName, &s, x - s.begin());
     return;
   }
@@ -514,7 +451,7 @@ Status JigdoIO::sectionEnd() {
   // Section that just ended was [Image]
   const char* valueName = 0;
   if (templateMd5 == 0) valueName = "Template-MD5Sum";
-  if (templateUrl.empty()) valueName = "Template";
+  if (templateUrls.isNull()) valueName = "Template";
   if (imageName.empty()) valueName = "Filename";
   if (valueName == 0) {
     imgSect_parsed();
@@ -531,7 +468,7 @@ Status JigdoIO::sectionEnd() {
 // "[Include url]" found - add
 void JigdoIO::include(string* url) {
   string includeUrl;
-  Download::uriJoin(&includeUrl, urlVal, *url);
+  uriJoin(&includeUrl, urlVal, *url);
   debug("%1:[Include %2]", line, includeUrl);
 
   JigdoIO* p = this;
@@ -543,23 +480,14 @@ void JigdoIO::include(string* url) {
     p = p->parent;
   } while (p != 0);
 
-  string leafname;
+  // childDl->source() is the source of the included .jigdo file's data
   auto_ptr<MakeImageDl::Child> childDl(
-      master()->childFor(includeUrl, 0, &leafname));
+      master()->childFor(includeUrl));
   if (childDl.get() != 0) {
-    MakeImageDl::IO* mio = master()->io.get();
-    string info = _("Retrieving .jigdo data");
-    string destDesc = subst(Job::MakeImageDl::destDescTemplate(),
-                            leafname, info);
-    auto_ptr<DataSource::IO> frontend(0);
-    if (mio != 0)
-      frontend.reset(mio->makeImageDl_new(childDl->source(), includeUrl,
-                                          destDesc) );
-    JigdoIO* jio = new JigdoIO(childDl.get(), includeUrl, frontend.get(),
+    // ...and jio is the destination of above data
+    JigdoIO* jio = new JigdoIO(childDl.get(), includeUrl, /*frontend.get(),*/
                                this, line);
-    childDl->setChildIo(jio);
-    frontend.release();
-    if (mio != 0) mio->job_message(&info);
+    childDl->source()->io.addListener(*jio);
 
     // Add new child
     JigdoIO** jiop = &firstChild;
@@ -574,7 +502,7 @@ void JigdoIO::include(string* url) {
 //______________________________________________________________________
 
 namespace {
-  // For Base64In - put decoded bytes into 16-byte array
+  /** Local class: For Base64In - put decoded bytes into 16-byte array */
   struct ArrayOut {
     typedef ArrayOut& ResultType;
     ArrayOut() { }
@@ -609,14 +537,14 @@ void JigdoIO::entry(string* label, string* data, unsigned valueOff) {
   } else if (section == "Jigdo") {
     if (*label == "Version") {
       if (value.empty()) return generateError(_("Missing argument"));
-      int ver = 0;
+      unsigned ver = 0;
       string::const_iterator i = value.front().begin();
       string::const_iterator e = value.front().end();
       while (i != e && *i >= '0' && *i <= '9') {
         ver = 10 * ver + *i - '0';
         ++i;
       }
-      if (ver > SUPPORTED_FORMAT)
+      if (ver > FILEFORMAT_MAJOR)
         return generateError(_("Upgrade required - this .jigdo file needs "
                                "a newer version of the jigdo program"));
     }
@@ -637,9 +565,8 @@ void JigdoIO::entry(string* label, string* data, unsigned valueOff) {
       imageName.assign(value.front(), lastSep + 1, 100);
       if (imageName.empty()) return generateError(_("Invalid image name"));
     } else if (*label == "Template") {
-      if (!templateUrl.empty()) return generateError(_("Value redefined"));
       if (value.empty()) return generateError(_("Missing argument"));
-      templateUrl = value.front();
+      master()->urlMap.addPart(urlVal, value, &templateUrls);
     } else if (*label == "Template-MD5Sum") {
       if (templateMd5 != 0) return generateError(_("Value redefined"));
       if (value.empty()) return generateError(_("Missing argument"));
@@ -669,10 +596,10 @@ void JigdoIO::entry(string* label, string* data, unsigned valueOff) {
       if (!imageInfo.empty()) return generateError(_("Value redefined"));
       imageInfo.assign(*data, valueOff, 5000);
     }
+    //____________________
 
   } else if (section == "Parts") {
 
-    // TODO
     if (value.empty()) return generateError(_("Missing argument"));
     MD5 md5;
     Base64In<ArrayOut> decoder;
@@ -689,7 +616,15 @@ void JigdoIO::entry(string* label, string* data, unsigned valueOff) {
         debug("x b64='%1' value='%2'", b64.result(), *label);
         return generateError(_("Invalid MD5Sum in Parts section"));
       }
-      debug("PART %1 -> %2", md5.toString(), value.front());
+      //debug("PART %1 -> %2", md5.toString(), value.front());
+      master()->urlMap.addPart(urlVal, md5, value);
+
+  } else if (section == "Servers") {
+
+    if (value.empty()) return generateError(_("Missing argument"));
+    const char* x = master()->urlMap.addServer(urlVal, *label, value);
+    if (x != 0) return generateError(x);
+
   } // endif (section == "Something")
 
 }
